use crate::serialization::{deserialize_str_at_end, DeserializeMessage, MessageType, serialize_message, SerializeMessage};

const CR_STACK_SIZE: usize = 2;

// Contains the version using semantic versioning
// The patch version is omitted, because it should not affect the communication between the server and the client
// Therefore, a client and a server with a different patch version should be completely compatible
pub struct ClientRegistration {
    pub major_version: u8,
    pub minor_version: u8,
    pub name: String,
}

impl SerializeMessage for ClientRegistration {
    fn get_message_type() -> MessageType {
        MessageType::NewConnection
    }

    fn serialize(&self) -> Vec<u8> {
        let mut buf: Vec<u8> = Vec::with_capacity(CR_STACK_SIZE);

        buf.insert(0, self.major_version);
        buf.insert(1, self.minor_version);
        buf.extend_from_slice(self.name.as_bytes());

        return serialize_message::<ClientRegistration>(&buf);
    }
}

impl DeserializeMessage for ClientRegistration {
    fn deserialize(buf: &[u8]) -> Self {
        ClientRegistration {
            major_version: buf[0],
            minor_version: buf[1],
            name: deserialize_str_at_end(&buf[2..]),
        }
    }
}
